@@ -3,7 +3,6 @@
 //! Start from [`Context`] struct documentation.
 //!
 //! [`libxm`]: https://github.com/Artefact2/libxm
-//! [`Context`]: Context
 //!
 //! ## Feature flags:
 //! - `defensive`: enables defensively checking the XM for errors/inconsistencies.
@@ -11,7 +10,11 @@
 //! - `ramping`: enables ramping (smooth volume/panning transitions, slightly increase CPU usage).
 //! - `strings`: enables module, instruments and sample names storing in context.
 
-use std::{ffi::CStr, mem::MaybeUninit};
+use std::{
+    ffi::CStr,
+    fmt::{self, Debug, Display, Formatter},
+    mem::MaybeUninit,
+};
 
 pub mod ffi;
 use ffi::*;
@@ -24,6 +27,20 @@ pub struct Context(ContextPointer);
 unsafe impl Send for Context {}
 unsafe impl Sync for Context {}
 
+impl Debug for Context {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Context")
+            .field("module_length", &self.module_length())
+            .field("patterns", &self.number_of_patterns())
+            .field("instruments", &self.number_of_instruments())
+            .field("channels", &self.number_of_channels())
+            .field("current_loop_count", &self.current_loop_number())
+            .field("speed", &self.playing_speed())
+            .field("position", &self.position())
+            .finish()
+    }
+}
+
 impl Context {
     /// Create a context.
     ///
@@ -35,6 +52,8 @@ impl Context {
     /// use exmod::Context;
     ///
     /// let context = Context::new(include_bytes!("module.xm"), 44100).unwrap();
+    ///
+    /// // And start working with it...
     /// ```
     pub fn new(module: &[u8], rate: u32) -> Result<Self, ContextError> {
         let mut pointer = MaybeUninit::uninit();
@@ -62,10 +81,10 @@ impl Context {
     /// Creates a context directly from the raw pointer of another context.
     ///
     /// # Safety
-    /// `pointer` must be previously allocated via [`new`] function or it's equivalent - [`xm_create_context_safe`].
+    /// `pointer` must be previously allocated via [`new`] and then [`as_ptr`] functions or their unsafe equivalent - [`xm_create_context_safe`].
     ///
     /// [`new`]: Self::new
-    /// [`xm_create_context_safe`]: xm_create_context_safe
+    /// [`as_ptr`]: Self::as_ptr
     pub unsafe fn from_ptr(pointer: ContextPointer) -> Self {
         Self(pointer)
     }
@@ -81,8 +100,8 @@ impl Context {
     /// See [`examples`] repo folder for comprehensive example.
     ///
     /// [`examples`]: https://gitlab.com/fluidex/exmod/-/tree/master/examples
-    pub fn generate_sample(&mut self, output: &mut [f32]) {
-        unsafe { xm_generate_samples(self.0, output.as_mut_ptr(), output.len() / 2) }
+    pub fn generate_samples(&mut self, samples: &mut [f32]) {
+        unsafe { xm_generate_samples(self.0, samples.as_mut_ptr(), samples.len() / 2) }
     }
 
     /// Get the current number of playback loops of the module.
@@ -93,7 +112,7 @@ impl Context {
     ///
     /// [`generate_sample`]: Self::generate_sample
     /// [`set_max_loop_count`]: Self::set_max_loop_count
-    pub fn loop_count(&self) -> u8 {
+    pub fn current_loop_number(&self) -> u8 {
         unsafe { xm_get_loop_count(self.0) }
     }
 
@@ -338,6 +357,17 @@ pub enum ContextError {
     Unknown(i32),
 }
 
+impl Display for ContextError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        match self {
+            #[cfg(feature = "defensive")]
+            Self::ModuleNotSane => f.write_str("Module not sane!"),
+            Self::MemoryAllocFailed => f.write_str("Memory allocation failed"),
+            Self::Unknown(number) => f.write_fmt(format_args!("Unknown error: {}!", number)),
+        }
+    }
+}
+
 /// Error of out of bounds.
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum OutOfBoundsError {
@@ -346,3 +376,14 @@ pub enum OutOfBoundsError {
     Channels,
     Samples,
 }
+
+impl Display for OutOfBoundsError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        f.write_str(match self {
+            Self::Patterns => "Pattern out of bounds!",
+            Self::Instruments => "Instrument out of bounds!",
+            Self::Channels => "Channel out of bounds!",
+            Self::Samples => "Sample out of bounds!",
+        })
+    }
+}
